// region.e
//
// Fri, 07/10/1992  17:42:35
//
// Purpose:
//
//		This extension modifies the Epsilon search routines to show
//		the current search match as a highlighted region.  The
//		highlighting behavior is modified by the new variable
//		search_shows_highlight which is non-zero if the search match
//		should be displayed.
//
// Usage:
//
//		Load this module and save your state file.
//
// The modifications to this code are dedicated to the public domain
// with the caveat that Lugaru is welcome to use this within their
// distribution source code which is supplied with Epsilon.  Use it
// any way you want for whatever purposes.  Portions are Copyright
// Lugaru Software...
//
//
//	John Kercheval
//	15563 10th Ave NE
//	Seattle, WA 98155-6209
//	INTERNET: jbk@wrq.com
//	Compu$erve: 72450,3702
//
//
// Modification History:
//
//		Fri, 07/10/1992  17:41:27 V1.0 initial release.
//
//		Thu, 07/08/1993  11:13:00 V1.1 AAC - made search-shows-highlight
//									  a user-modifiable global variable.
//									  Aaron A. Collins, CIS 70324,3200
//
// some amount of Epsilon original code so...
//
/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*   "EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.          *
*                                                                       *
*  Copyright (C) 1985, 1992 Lugaru Software Ltd.  All rights reserved.  *
*                                                                       *
* Limited permission is hereby granted to reproduce and modify this     *
* copyrighted material provided that the resulting code is used only in *
* conjunction with Lugaru products and that this notice is retained in  *
* any such reproduction or modification.                                *
************************************************************************/


#include "eel.h"

#ifndef BOOL
#define BOOL int
#define TRUE 1
#define FALSE 0
#endif


//
// This is the search handle used to show the search string
//
int search_region_handle = 0;

//
// This variable is TRUE (1) if the search string should be
// highlighted.
//
user int search_shows_highlight = TRUE;


////////////////////////////////////////////////////////////////////////////
//
// region_off removes the current search region highlight.
//

region_off()
{
	if (search_region_handle) { 
		remove_region(search_region_handle);
		search_region_handle = 0; 
	}
}


////////////////////////////////////////////////////////////////////////////
//
// new_show_conv_search will init and remove the search highlight
// region before and after, respectively, the search routine is
// called.
//

new_show_conv_search(flags)
{
	jmp_buf *old_level = top_level;
	jmp_buf this_level;
	int ret;

	//
	// remove the highlight if abort key is pressed or error occurs
	//
	top_level = &this_level;
	if (ret = setjmp(top_level)) {
		region_off();
		top_level = old_level;
		longjmp(top_level, ret);
	}

	show_conv_search(flags);

	region_off();
	top_level = old_level;
}

REPLACE_FUNC("show", "conv_search")


////////////////////////////////////////////////////////////////////////////
//
// new_show_gen_replace will init and remove the search highlight
// region before and after, respectively, the replace routine is
// called.
//

new_show_gen_replace(flags)
{
	jmp_buf *old_level = top_level;
	jmp_buf this_level;
	int ret;

	//
	// remove the highlight if abort key is pressed or error occurs
	//
	top_level = &this_level;
	if (ret = setjmp(top_level)) {
		region_off();
		top_level = old_level;
		longjmp(top_level, ret);
	}

	show_gen_replace(flags);

	region_off();
	top_level = old_level;
}

REPLACE_FUNC("show", "gen_replace")


////////////////////////////////////////////////////////////////////////////
//
// do_searching is a replacement of the routine by the same name in
// search.e in the Epsilon distribution and implements a region
// highlight for search finds.
//

do_searching(flags, str)	/* search according to flags */
char *str;	/* return 0=not found, 1=found, 2=aborted, 3=bad pattern */
{
	spot match_start = alloc_spot();
	spot match_end = alloc_spot();

	jmp_buf *old_level = top_level, this_level;
	int ret, dir = (flags & REVERSE) ? -1 : 1, ofold = case_fold;

	region_off();

	case_fold = (flags & FOLD);
	top_level = &this_level;
	if (setjmp(top_level))
		ret = DSABORT;
	else if (flags & REGEX)
		ret = do_regex_searching(flags, str);
	else if (flags & WORD)
		ret = word_search(dir, str);
	else
		ret = search(dir, str);
	top_level = old_level;
	case_fold = ofold;

	if (search_shows_highlight && ret == 1) {
		//
		// highlight the region (and thus the pattern match)
		//
		*match_start = matchstart;
		*match_end = matchend;
		search_region_handle = add_region(match_start, match_end,
										  color_class highlight, REGNORM);
		modify_region(search_region_handle, MRTYPE, REGNORM);
	}
	free_spot(match_start);
	free_spot(match_end);

	return ret;
}
